/* $Id: file-2.6.24,setup.c,v 1.6 2008/09/08 18:20:06 rupert Exp $
 *
 *  linux/arch/ppc/force/setup.c
 *
 *  Copyright (C) 1995  Linus Torvalds
 *  Adapted from 'alpha' version by Gary Thomas
 *  Modified by Cort Dougan (cort@cs.nmt.edu)
 *  Modified by Christian Zankel (zankel@mailserv.rz.fh-muenchen.de)
 *  Adapted for CES/RIO by Wolfgang Grandegger (wolfgang@ces.ch)
 */

/*
 * bootup setup stuff..
 */
#include <linux/autoconf.h>
#include <linux/init.h>
#include <linux/initrd.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/seq_file.h>
#include <linux/serial_8250.h>
#include <linux/proc_fs.h>

//#include <asm/bootinfo.h>
#include <asm/cache.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
//#include <asm/mk48t59.h>
#include <asm/mmu.h>
#include <asm/pgtable.h>
#include <asm/processor.h>
//#include <asm/residual.h>
#include <asm/sections.h>
#include <asm/time.h>
#include <asm/lmb.h>

#include "rio.h"
#include "rio_pic.h"
#include "rio_stubs.h"

#include <ces/cesResourceTable.h>
extern void cesRT_Dump(CesResourceTable_t const * table, int(*print)(const char *fmt, ...));

extern long __init rio_time_init(void);

extern void rio_find_bridges(void);
extern void rio_pci_request_resources(void);

extern unsigned long loops_per_jiffy;

#ifdef CONFIG_BLK_DEV_RAM
extern int rd_doload;		/* 1 = load ramdisk, 0 = don't load */
extern int rd_prompt;		/* 1 = prompt for ramdisk, 0 = don't prompt */
extern int rd_image_start;	/* starting block # of image */
#endif

#ifdef CONFIG_CES_DEBUG
extern void printf(const char *, ...);
#define dprintf printf
#else
#define dprintf(...) do {} while(0)
#endif

/* NVRAM */
unsigned char rio_nvram_rd (int addr)
{
	return in_8(rio_nvram_addr(addr, rio_board->cpu_nr));
}

void rio_nvram_wr (int addr, unsigned char val)
{
	out_8(rio_nvram_addr(addr, rio_board->cpu_nr), val);
}

/* FRAM */
/* Note, that the FRAM could also be accessed through the memory mapped
 * area, but this can cause latencies of 50 us (byte) up to 80 us (word)
 */
#define RIO_FRAM_REG ((void __iomem*)0xfa10c000)

#define RIO_FRAM_CMD_NONE  0x01000000
#define RIO_FRAM_CMD_WRITE 0x01000000
#define RIO_FRAM_CMD_READ  0x02000000
#define RIO_FRAM_CMD_RESET 0x03000000
#define RIO_FRAM_CMD       0x03000000

#define RIO_FRAM_WHILE_BUSY() do {} while(in_be32(RIO_FRAM_REG) & 0x80000000)
#define RIO_FRAM_COMMAND(cmd, addr, data) out_be32(RIO_FRAM_REG, RIO_FRAM_CMD_##cmd | ((addr) << 8) | data)
#define RIO_FRAM_DATA() (in_be32(RIO_FRAM_REG) & 0xff)

unsigned char rio_fram_rd (int addr)
{
	/* Wait until the device is idle */
	RIO_FRAM_WHILE_BUSY();
	/* Send the command */
	RIO_FRAM_COMMAND(READ, rio_nvram_offset(addr, rio_board->cpu_nr), 0);
	/* Wait again */
	RIO_FRAM_WHILE_BUSY();
	/* Get data */
	return RIO_FRAM_DATA();
}

void rio_fram_wr (int addr, unsigned char val)
{
	RIO_FRAM_WHILE_BUSY();
	RIO_FRAM_COMMAND(WRITE, rio_nvram_offset(addr, rio_board->cpu_nr), val);
}


//#define SHOW_L2CR
#define SHOW_HID0
#define SHOW_BATS
#define SHOW_MEMORY

#ifdef SHOW_BATS
extern
union ubat {			/* BAT register values to be loaded */
#ifdef CONFIG_PPC64BRIDGE
	u64	word[2];
#else
	u32	word[2];
#endif
} BATS[8][2];			/* 4 pairs of IBAT, DBAT */
extern
struct batrange {		/* stores address ranges mapped by BATs */
	unsigned long start;
	unsigned long limit;
	phys_addr_t phys;
} bat_addrs[8];
#endif

int rio_get_sdram_banks(struct seq_file *m);

void rio_show_cpuinfo(struct seq_file *m)
{
#define cpuinfo_bit(n,b) (((n & b)==b)?'1':'0')
#ifdef SHOW_L2CR
	static char *l2_siz[4]= { "unknown size", "256KB", "512KB", "1MB" };
	static char *l2_x[8]= {
		"noclock", "1x", "1.5x", "3.5x",
		"2.0x", "2.5x", "3.0x", "4.0x"
	};
	static char *l2_sync[4]= {
		"f/t sync burst",
		"reserved",
		"p/l sync burst",
		"p/l sync late-write"
	};
#endif
	static char rio3_pll_table[] = { 40, 60, 50, 66, 70, 90, 83, 100 };
	static char rio4_pll_table[] = { 40, 60, 50, 66, 70, 166, 133, 100 };
	char *pll_table = NULL;
	CesResourceTable_t const * restable;
	CesResourceTableEntry_t const * local_csr_entry;

	restable = cesRT_GetTable();
	local_csr_entry = cesRT_GetResource(restable, CESRT_LOCAL_CSR);

	if (cesRT_GetAddress(local_csr_entry) != CESRT_NONE && cesRT_GetVersion(local_csr_entry) == 0) {
		if (((cesRT_GetExtra(local_csr_entry) >> 4) & 3) == 0)
			pll_table = rio3_pll_table;
		else if (((cesRT_GetExtra(local_csr_entry) >> 4) & 3) == 1)
			pll_table = rio4_pll_table;
	}

	seq_printf(m, "machine\t\t: CES RIO%c %x %s Board (%s, SN %d)\n",
		(rio_board->cap & RIO_CAP_VME) ? ((rio_board->cap & RIO_CAP_RIO4) ? '4' : '3') : 'C',
		rio_board->model,
		(rio_board->cap & RIO_CAP_VME) ? "VME" : "CPCI",
		rio_board->name, rio_board->serial & 0xffff);

	if (!(rio_board->cap & RIO_CAP_RIO4)) {
		seq_printf(m, "FPGA Revision\t: %d\n", rio3_pic_get_rev());
	} else {
		const char *rio4_type;
		const char *rio4_type2 = "";
		const char *rio4_type3 = "";
		if (rio_board->cap & RIO_CAP_BIPRO) {
			rio4_type = "bipro";
			rio4_type3 = (rio_board->cpu_nr) ? " #A" : " #B";
		} else if (rio_board->cap & RIO_CAP_SIDEB)
			rio4_type = "sideb";
		else
			rio4_type = "sidea";
		if (rio_board->cap & RIO_CAP_VXS)
			rio4_type2 = " vxs";
		
		if (rio_board->cpu_nr == 0)
			seq_printf(m, "rio4 type\t: %s%s%s\n", rio4_type, rio4_type2, rio4_type3);
	}

#ifdef SHOW_L2CR
	{ /* L2CR */
		unsigned l2cr = mfspr(SPRN_L2CR);
		seq_printf(m, "L2 Cache\t: 0x%08x: %s, %s, %s, %s, %s\n",
			l2cr, /* (format: 0x%08x) */
			(l2cr & 0x80000000)?"enabled":"disabled",
			(l2cr & 0x40000000)?"parity":"noparity",
			l2_siz[(l2cr & 0x30000000) >> 28],
			l2_x[(l2cr & 0x0e000000) >> 25],
			l2_sync[(l2cr & 0x01800000) >> 23]);
	}
#endif
#ifdef SHOW_MEMORY
	{ /* memory */
		int total_size;
		seq_printf(m, "memory\t\t: ");
		total_size = rio_get_sdram_banks(m);
		seq_printf(m, "\n");
	}
#endif
#ifdef SHOW_HID0
	{ /* hid0 */
		unsigned hid0 = mfspr(SPRN_HID0);
		seq_printf(m, "HID0\t\t: %08x ICE=%c, DCE=%c, SGE=%c, BTIC=%c, BHTE=%c, BTCD=%c\n",
			 hid0, /*(format: "0x%08x, ") */
			cpuinfo_bit(hid0, HID0_ICE),
			cpuinfo_bit(hid0, HID0_DCE),
			cpuinfo_bit(hid0, HID0_SGE),
			cpuinfo_bit(hid0, HID0_BTIC),
			cpuinfo_bit(hid0, HID0_BHTE),
			cpuinfo_bit(hid0, HID0_BTCD));
	}
#endif
#ifdef SHOW_BATS
	{ /* bats */
		int i;
		int batcount = ppc_bat_count();
		seq_printf(m, "BAT __IBATH_ __IBATL_ __DBATH_ __DBATL_ __start_ __limit_ ______phys______\n");
		for (i = 0; i < batcount; i++) {
			seq_printf(m, " %d  %08x %08x %08x %08x %08lx %08lx %016Lx\n",
			           i,
			           BATS[i][0].word[0], BATS[i][0].word[1], 
			           BATS[i][1].word[0], BATS[i][1].word[1],
			           bat_addrs[i].start, bat_addrs[i].limit,
			           (unsigned long long)bat_addrs[i].phys);
		}
		
	}
#endif
	{ /* SUBIO_REG#3 */
		unsigned subio_reg3 = readl(RIO_LOC_REG3_ADRS);
		unsigned core_voltage;
		unsigned pci_p0_clk;
		unsigned pll_freq;

		if (((cesRT_GetExtra(local_csr_entry) >> 8) & 1) == 1) {
			pci_p0_clk = (subio_reg3 & 8) ? 25 : 33;
			seq_printf(m, "PCI_P0_CLK\t: %d MHz\n", pci_p0_clk);
		}
		if (((cesRT_GetExtra(local_csr_entry) >> 6) & 3) == 0) {
			core_voltage = 205 - ((subio_reg3 & 0xf0) >> 4) * 5;
			seq_printf(m, "Cpu_Vdd_Core\t: %d.%02d Volt\n",
				core_voltage / 100, core_voltage % 100);
		}
		if (pll_table) {
			pll_freq = (unsigned) pll_table[subio_reg3 & 7];
			seq_printf(m, "Clock_PLL_Cfg\t: %d MHz\n", pll_freq);
		}
	}
	/* FIXME:provide more information about the board */
}
#ifdef CONFIG_PHYS_64BIT
phys_addr_t fixup_bigphys_addr(phys_addr_t address, phys_addr_t size) {
	dprintf("%s()\n", __func__);
	return address;
}
#endif /* CONFIG_PHYS_64BIT */

struct vmemap_t {
	unsigned int vme_addr;
	unsigned int virt_addr;
} vmemap = {
	-1,
};

void __init rio_setup_io_mappings(void) {
	dprintf("%s()\n", __func__);
#ifdef RIO_MAP_COMPACT /* MAP SUBIO compact */
	dprintf("#### %s:%d: Compact Mapping!\n", __FUNCTION__, __LINE__);
/*
we may switch to allocating the resources (listed in init2) one-by-one,
and only the needed ones, to save address space.
*/
	u32 nvram_map_virt;
	u32 nvram_map_phys;
	u32 nvram_map_size;
	u32 csr_map_virt;
	u32 csr_map_phys;
	u32 csr_map_size;

#ifdef RIO_MAP_COMPACT_REALLY /* MAP SUBIO compact */
	nvram_map_size = RIO4_NVRAM_SIZE;
	nvram_map_virt = -nvram_map_size;
	nvram_map_phys = RIO_NVRAM_BASE_PHYS;
//	rio_board->nvram_base = (void __iomem*)nvram_map_virt;

	csr_map_size = RIO_LOC_REG_MEM_SIZE + RIO_PCI_REG_SIZE;
	csr_map_virt = nvram_map_virt - csr_map_size;
	csr_map_phys = RIO_LOC_REG_BASE_PHYS;
//	rio_board->csr_base = (void __iomem*)csrMmap_virt;
#else
#define MIN_BAT_SIZE 0x20000
#define MIN_BAT_MASK 0x1ffff
	/* NVRAM with this size also includes*/
	rio_board->nvram_base = (void __iomem*)((RIO_NVRAM_BASE_PHYS & MIN_BAT_MASK) - MIN_BAT_SIZE);
	nvram_map_virt = (u32)rio_board->nvram_base & ~MIN_BAT_MASK;
	nvram_map_phys = RIO_NVRAM_BASE_PHYS & ~MIN_BAT_MASK;
	nvram_map_size = MIN_BAT_SIZE;

	rio_board->csr_base = (void __iomem*)((RIO_LOC_REG_BASE_PHYS & MIN_BAT_MASK) - (MIN_BAT_SIZE * 2));
	csr_map_virt = (u32)rio_board->csr_base & ~MIN_BAT_MASK;
	csr_map_phys = RIO_LOC_REG_BASE_PHYS & ~MIN_BAT_MASK;
	csr_map_size = MIN_BAT_SIZE;
#endif

	io_block_mapping(nvram_map_virt, nvram_map_phys, nvram_map_size, _PAGE_IO);
	io_block_mapping(csr_map_virt, csr_map_phys, csr_map_size, _PAGE_IO);
	// The following areas will be required by boot code:
	   //-  1M @ 0xfa100000 - 0xfa1fffff NVRAM                         ?? real usage is smaller (64K)
	   //-  4K @ 0xf9008000 - 0xf9008fff SUBIO Registers
	   //-  4K @ 0xf9009000 - 0xf9009fff FPGA CSR Bus Registers
	 // The following areas are already mapped by their drivers
	   //-  8K @ 0xf900c000 - 0xf900dfff Serial                        ?? real usage is smaller
	   //- 64M @ 0xfc000000 - 0xffffffff Flash EPROM                   ?? real usage is smaller
	   //-  8K @ 0xf9000000 - 0xf9001fff SIC Interrupt Controller      ?? real usage is smaller
	   //- 16M @ 0xf8000000 - 0xf8ffffff Special D16 Flash Programming
	 // The following areas are not needed because there is no driver
	   //-  8K @ 0xf9004000 - 0xf9005fff Timer
	   //-  4K @ 0xf900b000 - 0xf900bfff Fifo Control Registers
	   //-  8K @ 0xf900e000 - 0xf900ffff FIFO Port Registers
#else
	io_block_mapping(0xf8000000, 0xf8000000, 0x08000000, _PAGE_IO);
#endif

#ifdef CONFIG_PHYS_64BIT
	if (rio_board->cap & RIO_CAP_BIPRO) {
		/* Map the memory of the other cpu */
		io_block_mapping(0x80000000, 0x400000000LL, 0x20000000, _PAGE_IO /* | _PAGE_USER*/);
		printk("RIO4: mapped 512 MB of the other CPU's memory at 0x%08x\n", 0x80000000);
	}
	if (rio_board->cap & RIO_CAP_RIO4) {
		char *found = strstr(cmd_line, "vmemap=");
		if (found && found[7] >= '0' && found[7] <= '9') {
			vmemap.vme_addr = simple_strtoul(found + 7, NULL, 0);
			vmemap.virt_addr = 0xa0000000;
			io_block_mapping(vmemap.virt_addr, (8ULL << 32) + vmemap.vme_addr, 0x10000000, _PAGE_IO | _PAGE_USER);
			printk("VMEMAP: mapped 256 MB of VME A32 0x%08x at virtual address 0x%08x\n", vmemap.vme_addr, vmemap.virt_addr);
		}
	}
#endif /* CONFIG_PHYS_64BIT */
}

static void *resource_table_seq_start(struct seq_file *s, loff_t *pos)
{

	if (*pos > CESRT_LAST_RESOURCE)
		return NULL;
	return (void*)(long)((*pos) + 1);
}

static void *resource_table_seq_next(struct seq_file *s, void *v, loff_t *pos)
{
	int id = (int)v;

	*pos = id + 1;
	if (id > CESRT_LAST_RESOURCE)
		return NULL;
	return (void*)(id+1);
}

static void resource_table_seq_stop(struct seq_file *s, void *v)
{
}

static int resource_table_seq_show(struct seq_file *s, void *v)
{
	int id;

	id = (int)v - 1;

	if (id == 0) {
		seq_printf(s, "CesResourceDescriptor_t table[] = {\n");
	} else if (id < CESRT_LAST_RESOURCE) {
		CesResourceTableEntry_t const * entry;
		entry = cesRT_GetResource(cesRT_GetTable(), id);
		if (entry->address != 0xffffffff) {
			seq_printf(s, "  {%3d, {0x%08x, 0x%08x, 0x%04x, 0x%04x, 0x%08x}},\n", id, (int)entry->address, entry->size, entry->version, entry->flags, entry->extra);
		}
	} else if (id == CESRT_LAST_RESOURCE) {
		seq_printf(s, "};\n");
	}

	return 0;
}

static struct seq_operations ct_seq_ops = {
	.start = resource_table_seq_start,
	.next  = resource_table_seq_next,
	.stop  = resource_table_seq_stop,
	.show  = resource_table_seq_show
};

static int resource_table_open(struct inode *inode, struct file *file)
{
	return seq_open(file, &ct_seq_ops);
};

static struct file_operations resource_table_fops = {
	.open    = resource_table_open,
	.read    = seq_read,
	.llseek  = seq_lseek,
	.release = seq_release
};

extern void init_rio_fpu_stats(void);

static void __init rio_init2(void){
	struct resource *res;
	CesResourceTable_t const * restable;
	CesResourceTableEntry_t const * entry;

	dprintf("%s()\n", __func__);
	init_rio_fpu_stats();

	res = request_mem_region(0xf8000000, 0x08000000, "SUB I/O Bridge");
	res->flags &= ~IORESOURCE_BUSY;

	request_mem_region(0, rio_board->memory_size, "System RAM");
	restable = cesRT_GetTable();
	entry = cesRT_GetResource(restable, CESRT_FRAM);
	if (cesRT_GetVersion(entry) == 1) {
		res = request_mem_region(RIO_NVRAM_BASE_PHYS, RIO4_NVRAM_SIZE, "RAMTRON FRAM FM32256");
		res->flags &= ~IORESOURCE_BUSY;
		request_mem_region(0xfa100000,     0x8000, "Direct memory mapped mode");
		request_mem_region(0xfa108000,     0x4000, "RTC");
		request_mem_region(0xfa10c000,     0x4000, "Register Mode Access");
	} else if (cesRT_GetVersion(entry) == 0) {
		request_mem_region(RIO_NVRAM_BASE_PHYS, RIO3_NVRAM_SIZE, "nvram/rtc");
	}
#ifdef CONFIG_PHYS_64BIT
	if (rio_board->cap & RIO_CAP_BIPRO)
		request_region(0x80000000, 0x20000000, rio_board->cpu_nr ? "CPU A" : "CPU B");
#endif
//	if (rio_board->cap & RIO_CAP_SSRAM)
//		request_region(0xfa000000,   0x100000, "SSRAM 128k x 32");
//	request_region(0xf900e000,     0x2000, "FIFO Port Registers");
// This will be requeseted by the serial driver
//	request_region(0xf900c000,     0x2000, "SIO Register");
//	request_region(0xf900b000,     0x1000, "FIFO Control");
	request_mem_region(0xf9009000,     0x1000, "FPGA CSR Bus");
	res = request_mem_region(0xf9008000,     0x1000, "SUBIO Registers");
	res->flags &= ~IORESOURCE_BUSY;
//	request_region(0xf9004000,     0x2000, "Timer");
	/* Disable all caches and reset the HIDs */
	if (rio_board->cap & RIO_CAP_XDIR) {
		res = request_mem_region(0xb8000000, 0x02000000, "SUB I/O XDIR High Speed Path");
		res->flags &= ~IORESOURCE_BUSY;
	}
	entry = cesRT_GetResource(restable, CESRT_SIC);
	if (cesRT_GetVersion(entry) == 1) {
		/* Map RIO4 SIC XSR space */
		request_mem_region(cesRT_GetAddress(entry), cesRT_GetSize(entry), "sic45");
	} else {
		request_mem_region(cesRT_GetAddress(entry), cesRT_GetSize(entry), "SIC Interrupt Controller");
	}
#ifdef CONFIG_PCI
	rio_pci_request_resources();
#endif
	{
		struct proc_dir_entry *entry;
		entry = create_proc_entry("CesResourceTable", S_IFREG, NULL);
		if (entry) {
			entry->proc_fops = &resource_table_fops;
		}
	}
}

#ifdef CONFIG_CES_DEBUG
void __init
rio_progress(char *s, unsigned short hex)
{
	printf("progress%8x: %s\n", hex, s);
}
#endif

void __init cesrt_early_serial_map(void) {
	struct uart_port port;
	int i;
	const CesResourceTable_t *resourceTable = cesRT_GetTable();

	dprintf("%s()\n", __func__);
	memset(&port, 0, sizeof(port));
	spin_lock_init(&port.lock);

	port.flags    = UPF_IOREMAP;
//	port.flags   |= UPF_BOOT_AUTOCONF;
//	port.flags   |= UPF_SHARE_IRQ;
//	port.flags   |= UPF_AUTO_IRQ;
//	port.flags   |= UPF_BUGGY_UART;

	port.iotype   = UPIO_MEM;
	port.regshift = 2;

	for (i = 0; i < 4; i++) {
		CesResourceTableEntry_t const * res;

		res = cesRT_GetResource(resourceTable, CESRT_SIO_0 + i);
		if (!cesRT_IsMy(res))
			continue;
		port.type    = PORT_16550A;
		switch(cesRT_GetVersion(res)) {
			case 0:
				port.uartclk = 16000000;
				break;
				
			case 1:
				port.uartclk = 16666667;
				break;
			case 2:
				port.uartclk = 3700000;
				break;
			default:
				printk(KERN_ERR "%s: unknown SIO version %d for port %d\n", __FUNCTION__, cesRT_GetVersion(res), i);
				continue;
		}
		port.mapbase = cesRT_GetAddress(res);
		port.membase = ioremap(port.mapbase, cesRT_GetSize(res));
		port.irq     = cesRT_GetExtra(res) & 0xff;
		early_serial_setup(&port);
		port.line++;
	}
}

void __init
rio_setup_arch(void)
{
	dprintf("%s() enter\n", __func__);
	cesrt_early_serial_map();

	/* init to some ~sane value until calibrate_delay() runs */
	loops_per_jiffy = 50000000;
#ifdef CONFIG_PCI
	rio_find_bridges();
#endif
	dprintf("%s() leave\n", __func__);
}

static void ppc_soft_restart(char *cmd)
{
	printk("%s: %s\n", __FUNCTION__, cmd);

	/* Disable irqs */
	local_irq_disable();

	/* Disable all caches and reset the HIDs */
	_set_L2CR(0);
	mtspr(SPRN_HID0, 0);
	mtspr(SPRN_HID1, 0);

	/* We put the new IP and MSR values into SRR[01]*/
	mtspr(SPRN_SRR0, 0xfff00100); /* NIP */
	mtspr(SPRN_SRR1, 0x00000040); /* MSR */

	/* Go! */
	__asm__ __volatile__ ("rfi");
}

void
rio_restart(char *cmd)
{
	int i=100000;
	char v;
	CesResourceTable_t const * restable;
	CesResourceTableEntry_t const * entry;
	restable = cesRT_GetTable();

	entry = cesRT_GetResource(restable, CESRT_LOCAL_CSR);

	printk("%s: %s\n", __FUNCTION__, cmd);

	/* soft reset of the CPU */
	local_irq_disable();

	/* set exception prefix high - to the prom */
	mtmsr(mfmsr() & ~MSR_IP);
//	_nmask_and_or_msr(0, MSR_IP);

	/* reset the H/W */
	if (((cesRT_GetExtra(entry) >> 9) & 1) == 1)
		v = RIO_LOC_REG0_RESET;
	else
		v = readb(RIO_LOC_REG0_ADRS) | RIO_LOC_REG0_RESET;

	writeb(v, RIO_LOC_REG0_ADRS);

	while (i--);
	panic("hard reset failed\n");
}

void
rio_halt(void)
{
/* FIXME: should we restart into the mon? */
	local_irq_disable();
	/* set exception prefix high - to the prom */
	mtmsr(mfmsr() | MSR_IP);
//	_nmask_and_or_msr(0, MSR_IP);
	for(;;);
}

/* Usually done by prom. */
void __init rio_init_lmb(void) {
	lmb_init();
	lmb_reserve(PHYSICAL_START, __pa(klimit) - PHYSICAL_START);
	lmb_add(0, rio_board->memory_size);
	lmb_enforce_memory_limit(memory_limit);
	lmb_analyze();
}

#ifdef CONFIG_SMP
extern struct smp_ops_t rio_smp_ops;
#endif

static rio_board_t rio_board_;
rio_board_t* rio_board = &rio_board_;
EXPORT_SYMBOL(rio_board);


void __init platform_init(void)
{
	CesResourceTable_t const * restable;
	CesResourceTableEntry_t const * entry;

	restable = cesRT_GetTable();
	dprintf("cesRT_GetCpuNumber() = %x\n", cesRT_GetCpuNumber());
	dprintf("cesRT_GetCpuFlag() = %x\n", cesRT_GetCpuFlag());
	dprintf("cur_cpu_spec->cpu_name  = %s\n", cur_cpu_spec->cpu_name);
	dprintf("cur_cpu_spec->pvr_mask  = %x\n", cur_cpu_spec->pvr_mask);
	dprintf("cur_cpu_spec->pvr_value = %x\n", cur_cpu_spec->pvr_value);
#ifdef CONFIG_CES_DEBUG
	cesRT_Dump(cesRT_GetTable(), printf);
#endif
	ISA_DMA_THRESHOLD = 0x0;
	DMA_MODE_READ = 0x44;
	DMA_MODE_WRITE = 0x48;
#ifdef CONFIG_PCI
	pci_dram_offset = rio_board->cpu_nr ? 0x20000000 : 0;
#endif
	rio_init_lmb();

	printk("\rRIO %x %s Board",
	        rio_board->model,
	       (rio_board->cap & RIO_CAP_VME) ? "VME" : "CPCI");
	if (rio_board->cap & RIO_CAP_BIPRO) {
		printk(" (#%c)", 'A' + rio_board->cpu_nr);
	}
	printk(" (%s, SN %d)\n",
	       rio_board->name, rio_board->serial & 0xffff);
#ifdef CONFIG_PCI
	printk("pci_dram_offset = %08lx\n", pci_dram_offset);
#endif

	ppc_md.setup_arch          = rio_setup_arch;
	ppc_md.show_cpuinfo        = rio_show_cpuinfo;
	
	entry = cesRT_GetResource(restable, CESRT_SIC);
	if (cesRT_GetVersion(entry) == 0) {
		ppc_md.init_IRQ            = rio3_pic_init;
	} else {
		ppc_md.init_IRQ            = rio4_pic_init;
	}

	entry = cesRT_GetResource(restable, CESRT_FRAM);
	if (cesRT_IsMy(entry)) {
		if (cesRT_GetVersion(entry) == 0 
		   || (cesRT_GetVersion(entry) == 1 && (cesRT_GetFlags(entry) & 0x8000))) {
			ppc_md.nvram_read_val      = rio_nvram_rd;
			ppc_md.nvram_write_val     = rio_nvram_wr;
		} else if (cesRT_GetVersion(entry) == 1) {
			ppc_md.nvram_read_val      = rio_fram_rd;
			ppc_md.nvram_write_val     = rio_fram_wr;
		}
	}

#ifdef CONFIG_CES_DEBUG
	ppc_md.progress            = rio_progress;
#endif

	ppc_md.init	           = rio_init2;

	ppc_md.restart	           = rio_restart;
	ppc_md.power_off           = rio_halt;
	ppc_md.halt	           = rio_halt;

#ifdef CALL_TIME_INIT_NOW
	rio_time_init();
	ppc_md.time_init           = NULL;
#else
	ppc_md.time_init           = rio_time_init;
#endif

	ppc_md.setup_io_mappings   = rio_setup_io_mappings;

	ppc_md.nvram_read_val      = rio_nvram_rd;
	ppc_md.nvram_write_val     = rio_nvram_wr;

#ifdef CONFIG_SMP
	ppc_md.smp_ops             = &rio_smp_ops;
#endif

#ifdef CONFIG_VT
	/* no keyboard */
#error RIO does not support a keyboard
#endif
}

